library telescope;

// DLL pour MCMT avec firmware "MCMTII"
// De faon gnrale, la communication se fera par le port COM. Ce port sera ouvert
//  chaque envoi de commande et ferm juste aprs !

{$ALIGN 8}

(* Changes record
  Aug 2001 : BEWARE OF ALIGNMENT !!
             Delphi6 is A8 by default, Delphi5 is A4
             Aowner in InitCodeurs may cause problems for non Delphi languages, in that case leave it to NIL and
             use file or registry to set up the hardware.

  Oct 2001 : CHANGED from
             Function InitCodeurs(Var TimeCheck:Integer;Aowner:Tcomponent;Var EnCodersCaps:TEnCodersCaps):boolean; stdcall;
             To
             Function InitCodeurs(Var TimeCheck:Integer;Aowner:Tcomponent;Latitude,Longitude:Double):boolean; stdcall;


  janv 2004 : No setup panel poping-up during InitCodeurs procedure !

              added :
                Init_telescope_Panel(Aowner:Tcomponent;TwinHandle:Thandle;TypeSetup:TsetupEngine_encoderEnumerator); stdcall;

               To allow setup panel to be called by PRISM setup telescope...
*)

uses
  forms,
  Dialogs,
  SysUtils,
  Math,
  Classes,
  Controls,
  Windows,
  IniFiles,
  Registry,
  Console in 'Console.pas' {FormConsole},
  Setup in 'Setup.pas',
  COMM in 'COMM.pas',
  Utils in 'Utils.pas',
  speed in 'speed.pas' {SpeedEngine},
  scan_mcmt in 'scan_mcmt.pas' {Form_scan_mcmt};

{$R *.RES}

Var GDelta,GAlpha: Double ;    // Retranscrit les positions en AD et DEC des codeurs

    //Composant:TComponent;    // Ncessaire pour ouvrir des fiches depuis PRiSM

Const
  ERROR   =False;
  NOERROR =True;

// stdcall = to be compliant with C/C++
(* Warning : it could be possible that single hardware is used to access
   encoders, telescope pointing, focusing : in that case care must taken
   during acces to this hardware (ie COM for instance) to avoid
   multiple acces and conflicts.

   In some cases, encoders, telescope, ect could have completely different ways to be
   accessed... ie telescope : thru IP stack; encoders thru COM1 etc etc
*)

Var
   Resol_AD,Resol_DEC : longint;        // Rsolution des codeurs
   AlphaRaw,DeltaRaw,AlphaRawOld,AlphaRawInit : longint;         // Lecture codeurs en upas
   AlphaInit,DeltaInit,AlphaTarget, DeltaTarget,AlphaOldPoint,DeltaOldPoint: Double ;        // Valeurs initiales pour calibration
   AlphaCodeurInit,DeltaCodeurInit: Double ;  // Valeurs initiales Codeurs pour calibration
   FirstTime,delta_time,atime,Long,Lat: Double ;  // Valeurs initiales pour calibration
   Inv_Codeur_AD,Inv_Codeur_DEC,KING_Activated,PEC_Activated,Park: boolean;
   V_Guidage_AD,V_Guidage_AD_SID,V_Guidage_DEC,V_Corr_P_AD,V_Corr_P_DEC,V_Corr_M_AD,V_Corr_M_DEC,
   V_Point_L_AD,V_Point_L_DEC,V_Point_R_AD,V_Point_R_DEC,PEC_Step_Init,PEC_Step_Codeur_Arcsec : Double;
   Amplitude_PEC,Phase_PEC,Periode_PEC: array[1..2] of double;
   Accel_AD,PEC_Step,GMT,PEC_Step_old,FCorrectionPeriodeMs: integer;

//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
// General
// Allocation must be done by the DLL

Function GetDLLVer:Pchar;  stdcall;

var S,Inter  : string;
    n, Len   : DWORD;
    Buf      : PChar;
    Value    : PChar;

Begin
 Try

   Inter:='MCMTII DLL version ';

   S    := 'telescope.dll';
   n := GetFileVersionInfoSize(PChar(S), n);

   if n > 0 then
   begin
    Buf := AllocMem(n);
    GetFileVersionInfo(PChar(S), 0, n, Buf);
    if VerQueryValue(Buf, PChar('StringFileInfo\040C04E4\FileVersion'),Pointer(Value),Len) then
      Inter:=Inter+Value;
    FreeMem(Buf, n);
    GetDLLVer:=PChar(Inter);
   end
   else
   GetDLLVer:=PChar('OMCMTII DLL version = ???');

 Except
   GetDLLVer:=PChar('MCMTII DLL version = Error !');
 end;
end;

//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
// Codeur

Function Calibrate(Alpha,Delta:Double):boolean; stdcall;
 (* Renseignements Cyril :
   This function recalibrates the encoder position to Alpha,Delta. Either sent to Hardware or internally handled
   by this DLL... result as true for success*)
Var
  Err_COM:boolean;
  astring: string;

begin
//Vrification des ports COM

// while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
 AccesCom(NumCOM);

 try
//   Open_Cod_Eng(Err_COM);  // ouvrir le port COM
   try
    if deviceready(0) = 1 then
    begin
     //Alpha et Delta sont en RADIANS !!!
     //Transformation Alpha
     While Alpha <  0    do Alpha:=Alpha+(2*Pi);
     While Alpha >= 2*Pi do Alpha:=Alpha-(2*Pi);   // Alpha est alors compris dans [0;2*Pi]

     AlphaCodeurInit :=  (ReadEncoder(3,0) shl 24) + (ReadEncoder(2,0) shl 16) + (ReadEncoder(1,0) shl 8) + ReadEncoder(0,0);
     AlphaRawInit := round(AlphaCodeurInit);
     AlphaCodeurInit := 2*Pi*frac(AlphaCodeurInit / Resol_AD);

     DeltaCodeurInit :=  (ReadEncoder(3,1) shl 24) + (ReadEncoder(2,1) shl 16) + (ReadEncoder(1,1) shl 8) + ReadEncoder(0,1);
     DeltaCodeurInit := 2*Pi*frac(DeltaCodeurInit / Resol_DEC);

     AlphaInit := Alpha;
     DeltaInit := Delta;
     if DeltaInit < 0 then
        DeltaInit := 2*Pi + DeltaInit;
     GAlpha := Alpha;
     GDelta := Delta;

     FirstTime := Time;
     atime := Time;
     result:=NOERROR;
     if Park then
     begin
      astring:='N'+chr(0)+chr(0)+chr(0)+chr(0);                      // Commande No Park
      sendcommand(astring,0,0);
      Park := false;
     end;
    end
    else
     begin
      result:=ERROR;
     end;
   finally
//     CloseHandle(hCom);
   end;
 finally
   AccesCom(NumCOM);
 end;
end;

Function InitCodeurs(Var TimeCheck:Integer;Aowner:Tcomponent;Latitude,Longitude:Double):boolean; stdcall;
 (* Renseignements Cyril :
   This function should initialise the encoder device, if it fails it should return
   false as a result.
   This function has to display ITSELF a form to ask eventually about :
    COM, Ip address, initialisation parameters, etc etc....

   It should provide the time (in ms) where PRISM refreshes internaly Alpha/Delta, do put something
   between 0.5 to 2 sec, too high figures could degrade pointing accuracy, too low figure could
   have an impact on general CPU performances.

 *)
Var
  Form_scan_mcmt:TForm_scan_mcmt;
  DeviceIni: TIniFile;
  c,retour: integer;
  found: boolean;
  TimeZone: TTimeZoneInformation;
  ValPark_AD,ValPark_DEC: double;
  TmpPorts: TStringList;

Begin
 Long := Longitude;
 long_Park := Long;
 Lat := Latitude;

 retour := GetTimeZoneInformation(TimeZone);
 Case retour Of
    TIME_ZONE_ID_UNKNOWN  : GMT := TimeZone.Bias div 60;
    TIME_ZONE_ID_STANDARD : GMT := (TimeZone.Bias+TimeZone.StandardBias) div 60;
    TIME_ZONE_ID_DAYLIGHT : GMT := (TimeZone.Bias+TimeZone.DaylightBias)div 60;
  Else
    Exit;
  End;

 GMT_Park := GMT;

 decimalseparator:='.';

 DeviceIni := TIniFile.Create('mcmtii.ini');
   with DeviceIni do
   begin
    NumCOM:=ReadInteger('Configuration','Port',1);
    GDelta:=strtofloat(ReadString('Configuration','LastDelta','0'));
    GAlpha:=strtofloat(ReadString('Configuration','LastAlpha','0'));
    Inv_Codeur_AD := ReadBool('Parameters', 'DIR_AD', False);
    Inv_Codeur_DEC := ReadBool('Parameters', 'DIR_DEC', False);
    Resol_AD:= ReadInteger('Parameters', 'Resol_AD', 4608000);
    Resol_DEC:= ReadInteger('Parameters', 'Resol_DEC', 4608000);
    V_Guidage_AD := strtofloat(ReadString('Parameters', 'V_Guidage_AD', '1'));
    V_Guidage_AD_SID := V_Guidage_AD;
    V_Guidage_DEC := strtofloat(ReadString('Parameters', 'V_Guidage_DEC', '1'));
    V_Corr_P_AD := strtofloat(ReadString('Parameters', 'V_Corr_P_AD', '1'));
    V_Corr_P_DEC := strtofloat(ReadString('Parameters', 'V_Corr_P_DEC', '1'));
    V_Corr_M_AD := strtofloat(ReadString('Parameters', 'V_Corr_M_AD', '1'));
    V_Corr_M_DEC := strtofloat(ReadString('Parameters', 'V_Corr_M_DEC', '1'));
    V_Point_L_AD := strtofloat(ReadString('Parameters', 'V_Point_L_AD', '1'));
    V_Point_L_DEC := strtofloat(ReadString('Parameters', 'V_Point_L_DEC', '1'));
    V_Point_R_AD := strtofloat(ReadString('Parameters', 'V_Point_R_AD', '1'));
    V_Point_R_DEC := strtofloat(ReadString('Parameters', 'V_Point_R_DEC', '1'));
    Accel_AD := strtoint(ReadString('Parameters', 'Accel_AD', '0'));
    Park := ReadBool('ParkMode','Park',False);
    ValPark_AD:=Readfloat('ParkMode','AH',0);
    ValPark_DEC:=Readfloat('ParkMode','DEC',0);
   end;

 if Park then
   begin
    GAlpha := HeureSiderale(long_Park,GMT_Park) - ValPark_AD;
    GDelta := ValPark_DEC;
    with DeviceIni do WriteBool('ParkMode','Park',False);
   end;

 DeviceIni.free;

 found := false;
 stopscan := false;

 TmpPorts := TStringList.Create;
 FormConsole:=TFormConsole.create(Aowner);  // To pass Aowner is VERY important
 EnumComPorts(TmpPorts);

 try                      // on test si MCMT prsent  la mme com qu'auparavant
  Open_Cod_Eng(Err_COM);  // ouvrir le port COM
  try
   if deviceready(0) = 1 then
    begin
     found := true;
     if deviceready(2) = 1 then
         begin
          Focus_enabled := true;
          FormConsole.Focus.Visible := true;
         end
        else
         FormConsole.Focus.Visible := false;
    end;
  finally
   CloseHandle(hCom);
  end;
  except;
 end;

 if not found then        // Sinon on scan sur les autres ports dispo
 begin
  Form_scan_mcmt:=TForm_scan_mcmt.create(Aowner);
  Form_scan_mcmt.show;
  c:=0;
  while (c < TmpPorts.Count) and (found = false)
  do
   begin
    NumCom := strtoint(TmpPorts[c]);
    Form_scan_mcmt.Panel_Info.Caption := 'Recherche MCMT sur com ' + inttostr(NumCOM);
    try
     Open_Cod_Eng(Err_COM);  // ouvrir le port COM
     try
      if deviceready(0) = 1 then
       begin
        found := true;
        c:=TmpPorts.Count;
        if deviceready(2) = 1 then
         begin
          Focus_enabled := true;
          FormConsole.Focus.Visible := true;
         end
        else
         FormConsole.Focus.Visible := false;
       end;
      if stopscan = true then c:=TmpPorts.Count;
     finally
      CloseHandle(hCom);
     end;
    except;
    end;
     c := c + 1;
   end;
  Form_scan_mcmt.free;
 end;

 TmpPorts.Free;

   if Not found then
   begin
      if not stopscan then
         ShowMessage('Le programme n''a pas dtect MCMT !');
      result:= ERROR;
      exit;
   end
   else
   begin
      Open_Cod_Eng(Err_COM);  // ouvrir le port COM
   end;


 // Initialisation de Delta et Alpha avec dernires valeurs enregistres

  if Calibrate(GAlpha,GDelta) = false then
      begin
       result:= ERROR;
       ShowMessage('Erreur de COM, le systme ne rpond pas!');
      end
     else
      result:= NOERROR;


  //KING_Activated := true;

  //Fixer TimeCheck
  TimeCheck:=1000;      // 1s parat bien; c'est un minimum mais pas trop souvent quand mme
                        // pour ne pas saturer le processeur !
  FCorrectionPeriodeMs := TimeCheck;
end;

Function CloseCodeurs:boolean; stdcall;
(* Renseignements Cyril :
   This function closes the driver and eventually save data to registery or/file.
   If it fails sould return false as result.
 *)

Var
   DeviceIni: TIniFile;

begin
   CloseHandle(hCom);
   DestroySectionCritique;
   DeviceIni := TIniFile.Create('mcmtii.ini');
   with DeviceIni do
   begin
      WriteString('Configuration','LastDelta',floattostr(GDelta));
      WriteString('Configuration','LastAlpha',floattostr(GAlpha));
   end;
   DeviceIni.free;
   FormConsole.free;
   result:=True;
end;

Function getAlpha(Var Alpha:Double):boolean; stdcall;
 begin
 (* Renseignements CYRIL :
   This function should stay as it is
   The Galpha value is refreshed by RetrieveDataAlphaDelta every
   TimeCheck Ms
   Bah alors, on n'y touche pas !
 *)
  result:=True;
  Alpha:=GAlpha;
end;


Function getDelta(Var Delta:Double):boolean; stdcall;
 begin
 (*
   This function should stay as it is
   The Galpha value is refreshed by RetrieveDataAlphaDelta every
   TimeCheck Ms
   Bah alors, on n'y touche pas !
 *)
  result:=True;
  Delta:=GDelta;
end;



Function RetrieveDataAlphaDelta:boolean; stdcall;

Var
   Err_COM:boolean;
   AlphaTemp,DeltaTemp:Double;
   readtemp,i,c,valeur: integer;
   astring,receivestring:ShortString;
   SIGNE,MSB,LSB,LLSB: byte;
   aval,tempPEC,position,longi,temp,H,M,S,U: Double;
   DeviceIni : TIniFile;
   achar: char;


begin
  (* Renseignements CYRIL :
   This function is called every TimeCheck Ms by a threaded timer internal to PRISM
   This is MANDATORY to fill up this feature with something :-) because it gives
   values to GDelta and GAlpha . This is called by a process, be careful if hardware
   is shared by other devices such as telescope pointing, focusing.
   Make sure that the hardware ressource is FREE before acessing to it !!*)

//Vrifier le port COM
// while not AccesComAllowance(NumCOM) do application.ProcessMessages;
  AccesCom(NumCOM);
 try
//  Open_Cod_Eng(Err_COM);  // Ouvrir le port COM
  try
   if(true) then
   begin
    readtemp := ReadEncoder(3,0);
    atime := Time;
    while(atime < FirstTime) do atime := atime + 24;
    delta_time:=frac(atime-FirstTime)*24*15*Pi*86400/(86164.101*180);
     //delta_time:=delta_time+24*15*Pi/(86164.101*180);
     if readtemp = -1 then                  //MCMT toujours branch?
        begin
         result:=error;                     //Quitte la fonction avec erreur
         exit;
        end;
     AlphaRaw :=  (readtemp shl 24);
     DeltaRaw :=  (ReadEncoder(3,1) shl 24);
     AlphaRaw := AlphaRaw + (ReadEncoder(2,0) shl 16);
     DeltaRaw := DeltaRaw + (ReadEncoder(2,1) shl 16);
     AlphaRaw := AlphaRaw + (ReadEncoder(1,0) shl 8);
     DeltaRaw := DeltaRaw + (ReadEncoder(1,1) shl 8);
     AlphaRaw := AlphaRaw + ReadEncoder(0,0);
     DeltaRaw := DeltaRaw + ReadEncoder(0,1);

     if Focus_enabled then
     begin
      FocusRaw :=  (ReadEncoder(3,2) shl 24);
      FocusRaw := FocusRaw + (ReadEncoder(2,2) shl 16);
      FocusRaw := FocusRaw + (ReadEncoder(1,2) shl 8);
      FocusRaw := FocusRaw + ReadEncoder(0,2);
      FocusPosition := FocusRaw - FocusInit;
      FormConsole.Panel_Position.caption := inttostr(FocusPosition);
     end;

     FormConsole.edit3.text := floattostr((AlphaRaw-AlphaRawInit)/(frac(atime-FirstTime)*24*3600));
     AlphaRawOld := AlphaRaw;

     AlphaTemp := 2*Pi * frac(AlphaRaw / Resol_AD);

     //AlphaOld := GAlpha;

     if not Inv_Codeur_AD then
        GAlpha:=2*Pi*frac((AlphaInit+AlphaTemp-AlphaCodeurInit+delta_time) / (2*Pi))
     else
        GAlpha:=2*Pi*frac((AlphaInit-AlphaTemp+AlphaCodeurInit+delta_time) / (2*Pi));

     While GAlpha >= 2*Pi do GAlpha:=GAlpha-(2*Pi);   // GAlpha est alors compris dans [0;2*Pi]

     DeltaTemp := 2*Pi*frac(DeltaRaw / Resol_DEC);

     //DeltaOld:= GDelta;

     if Inv_Codeur_DEC then
        GDelta := DeltaInit + DeltaTemp - DeltaCodeurInit
     else
        GDelta :=  - DeltaTemp + DeltaInit + DeltaCodeurInit;

     GDelta := 2*Pi*frac(GDelta / (2*Pi));

     If GDelta < 0 then
        GDelta := 2*Pi + GDelta;

     {if (abs(GDelta - DeltaOld) > pi/8)
        then
         GDelta := DeltaOld; }

     if (GDelta > (Pi/2)) and (GDelta < (3*Pi/2)) then
     begin
      //if (GDelta > (Pi/2)) and  (GDelta  < Pi) then
       GDelta := Pi - GDelta;
      //else
       //GDelta := GDelta - Pi;
      GAlpha := GAlpha + Pi;
      While GAlpha >= 2*Pi do GAlpha:=GAlpha-(2*Pi);   // GAlpha est alors compris dans [0;2*Pi]
     end;
     if GDelta >= (3*Pi/2) then
        GDelta := GDelta - 2*Pi;

     While GAlpha <  0    do GAlpha:=GAlpha+(2*Pi);
     While GAlpha >= 2*Pi do GAlpha:=GAlpha-(2*Pi);   // GAlpha est alors compris dans [0;2*Pi]

     {if (abs(GAlpha - AlphaOld) > pi/8) and (abs(GAlpha - AlphaOld) < 3*pi/2)
        then
         GAlpha := AlphaOld;}

     longi := -long*180/pi;

     aval := V_Guidage_AD;
     FormConsole.GuideStr.text := floattostr(aval);

     AH_Park := AH(longi,Galpha,GMT);
     GAlpha_Park := GAlpha;
     GDelta_Park := GDelta;

    if (Focus_enabled) and (FocusCommand) then
     begin
       if FocusStr = 'StopSlewing' then
       begin
        StopSlewing(2);
        FocusCommand := false;
       end
       else
       if deviceready(2) = 1 then
       begin
        FocusCommand := false;
        sendcommand(FocusStr,0,2);
       end;
     end;

    if deviceready(0) = 1 then
     begin
      astring := 'r'+chr(0)+chr(0)+chr(0)+chr(0);
      receivestring := sendcommand(astring,10,0);
        if receivestring <> '' then
         if (ord(receivestring[2]) <> 0) or (ord(receivestring[1]) <> 0) then
          begin
           FormConsole.VitesseStr.text := floattostrf(625000/(ord(receivestring[3])/10+ord(receivestring[2])+256*ord(receivestring[1])),ffFixed,6,6);
           FormConsole.Panel_Index_Pec.caption := inttostr(ord(receivestring[4]));
           PEC_Step := ord(receivestring[6])+256*ord(receivestring[5]);
           if not Inv_Codeur_AD then
                PEC_Step := 25600 - PEC_Step;
           FormConsole.Panel_compteur_Pec.caption := inttostr(PEC_Step);
           readtemp := 2*(ord(receivestring[8])+256*ord(receivestring[7]));
           if not Inv_Codeur_AD then
                readtemp := 25600 - readtemp;
           FormConsole.Panel_prochaine_correction.caption := inttostr(readtemp);
          end;

          case (ord(receivestring[9]) and 15) of
           0:  FormConsole.Panel_info_pec.caption := 'Correction MCMT des erreurs priodiques inactive';
           1:  FormConsole.Panel_info_pec.caption := 'Correction MCMT des erreurs priodiques active';
           2:  FormConsole.Panel_info_pec.caption := 'En attente du passage  zro pour synchronisation!';
           3:  FormConsole.Panel_info_pec.caption := 'Enregistrement des corrections en cour!';
          end;

         if ord(receivestring[10]) = 1 then  FormConsole.VitesseStr.text := 'PARK!';

         FormConsole.Panel_vitesse.caption := FormConsole.VitesseStr.text;

         if ord(receivestring[9]) < 2 then
         begin
          FormConsole.ButtonActiverPEC.Enabled := true;
          if ord(receivestring[9]) = 0 then
          begin
           FormConsole.ButtonActiverPEC.caption := 'Activer';
           FormConsole.ButtonEnregistrerPEC.Enabled := true;
           FormConsole.ButtonEffacerMem.Enabled := true;
           FormConsole.ButtonSauverPEC.Enabled := true;
           FormConsole.ButtonCHargerPEC.Enabled := true;
          end
          else
           begin
            FormConsole.ButtonActiverPEC.caption := 'Dsactiver';
            FormConsole.ButtonEnregistrerPEC.Enabled := false;
            FormConsole.ButtonEffacerMem.Enabled := false;
            FormConsole.ButtonSauverPEC.Enabled := false;
            FormConsole.ButtonCHargerPEC.Enabled := false;
           end;
         end
         else
          begin
           FormConsole.ButtonActiverPEC.caption := 'Dsactiver';
           FormConsole.ButtonActiverPEC.Enabled := false;
           FormConsole.ButtonEnregistrerPEC.Enabled := false;
           FormConsole.ButtonEffacerMem.Enabled := false;
           FormConsole.ButtonSauverPEC.Enabled := false;
           FormConsole.ButtonCHargerPEC.Enabled := false;
          end;

         FormConsole.Edit1.Text := floattostr(strtofloat(FormConsole.VitesseStr.text)/strtofloat(FormConsole.GuideStr.text));
         temp:=AH(longi,Galpha,GMT)*12/pi;
         //temp:=HeureSiderale(longi,GMT)*12/pi;
         H:=int(temp);
         M:=int((abs(temp)-abs(H))*60);
         S:=60*frac((abs(temp)-abs(H))*60);
         FormConsole.Edit2.Text:=inttostr(round(H))+':'+inttostr(round(M))+':'+floattostr(S);

         if((KING_Activated) or (PEC_Activated)) and (deviceready(0) = 1) then
          begin
           if(King_Activated) then
            begin
             aval := aval / getKingSpeedCorrection(AH_Park,GDelta,Lat);
            end;
           if(PEC_Activated) then
            begin
             position := Periode_PEC[1] * frac(PEC_Step / 25600);
             FormConsole.Pec_posi.text := inttostr(round(position));

             // This is the derivative function of X + A1*cos(2Pi/Per*(X + Phi1)) + A2*cos(2Pi/Per*(X + Phi2))

             tempPEC:= 0;
             For i:=1 to 2 do
              begin                                                                                                                    //(Time*Arcsecc)
               U:= (Pi/Periode_PEC[i]) * (2*Amplitude_PEC[i]) * sin( ((2*Pi)/Periode_PEC[i])  * ( i * (position + Phase_PEC[i]) + (15.041 * FCorrectionPeriodeMs / 1000)) );
               tempPEC := tempPEC + U;
              end;

                //(Time*Arcsecc) o Time Corrige l'avance de phase par anticipation

                // Sinus derivative is negative !
              tempPEC := 1 - tempPEC ;

              aval := aval * tempPEC;
            end;
            aval := 625000 / aval;
            MSB:= (round(abs(aval)) SHR 8) and $FF;
            LSB:= (round(abs(aval))) and $FF;
            LLSB:= round(abs(10 * frac(aval))) and $FF;
            SIGNE := 0;
            if aval > 0 then SIGNE := 1;
            astring:='R'+chr(MSB)+chr(LSB)+chr(LLSB)+chr(SIGNE);
            sendcommand(astring,0,0);
          end;

         if ActiverPEC then
         begin
          ActiverPEC := false;
          astring := 'L'+chr(47)+chr(1)+chr(0)+chr(0);
          receivestring := sendcommand(astring,0,0);
         end;

         if DesactiverPEC then
         begin
          DesactiverPEC := false;
          astring := 'L'+chr(47)+chr(0)+chr(0)+chr(0);
          receivestring := sendcommand(astring,0,0);
         end;

         if EffacerPEC then
         begin
          EffacerPEC := false;
          astring := 'e'+chr(0)+chr(0)+chr(0)+chr(0);
          receivestring := sendcommand(astring,0,0);
         end;

         if EnregistrerPEC then
         begin
          EnregistrerPEC := false;
          astring := 'E'+chr(0)+chr(0)+chr(0)+chr(0);
          receivestring := sendcommand(astring,0,0);
         end;

         if ChargerPEC then
         begin
          ChargerPEC := false;
          if FormConsole.OpenDialog2.Execute then          // Display Save dialog box
          begin
           DeviceIni := TIniFile.Create(FormConsole.OpenDialog2.FileName);
           i:=49;
           c:=1;
           while i<249 do
           begin
            with DeviceIni do valeur := ReadInteger('PEC', 'BASE_'+inttostr(c), 0);
            while deviceready(0) <> 1 do ; //Application.ProcessMessages;
            astring:='L'+chr(i)+chr(valeur)+chr(valeur shr 8)+chr(0);
            sendcommand(astring,0,0);
            i := i + 2;
            c:= c + 1;
           end;
           DeviceIni.free;
          end;
         end;

         if SauverPEC then
         begin
          SauverPEC := false;
          if FormConsole.SaveDialog1.Execute then          // Display Save dialog box
          begin
           DeviceIni := TIniFile.Create(FormConsole.SaveDialog1.FileName);
           i:=49;
           c:=1;
           while i<249 do
           begin
            if deviceready(0) = 1 then
            begin
             astring := 'J'+chr(i)+chr(0)+chr(0)+chr(0);
             receivestring := sendcommand(astring,1,0);
             achar := receivestring[1];
             valeur := ord(achar);
            end;
            if deviceready(0) = 1 then
            begin
             astring := 'J'+chr(i+1)+chr(0)+chr(0)+chr(0);
             receivestring := sendcommand(astring,1,0);
             achar := receivestring[1];
             valeur := valeur + 256 * ord(achar);
            end;
            with DeviceIni do WriteInteger('PEC', 'BASE_'+inttostr(c) , valeur);
            i := i + 2;
            c:= c + 1;
           end;
           DeviceIni.free;
          end;
         end;

         if GoPark then
         begin
          GoPark := false;
          while deviceready(0) <> 1 do ; //Application.ProcessMessages;
          astring:='P'+chr(0)+chr(0)+chr(0)+chr(0);
          sendcommand(astring,0,0);
         end;

     end;

   end;

  finally
//   CloseHandle(hCom);
  end;
 finally
  AccesCom(NumCOM);
 end;
 result := true;
 if NOT true then showmessage('erreur com');
end;

Function AcceptRawPositionCodeurs:Boolean;stdcall;
Begin
 result:=NOError;
End;

Function GetRawPositionAlphaInit_arcsecCodeurs:Double;stdcall;
begin
 PEC_Step_Old := PEC_Step;
 PEC_Step_Codeur_Arcsec := 3600 * (9216000 / Resol_AD) * frac(PEC_Step / 25600);
 result := 0;
End;

Function GetRawPositionDeltaInit_arcsecCodeurs:Double;stdcall;
Begin
 result:=0;
End;

Function GetRawPositionAlpha_arcsecCodeurs:Double;stdcall;

var
 temp: double;

Begin
 if PEC_Step < PEC_Step_Old then
  temp :=  PEC_Step + 25600 - PEC_Step_Old
 else
  temp :=  PEC_Step - PEC_Step_Old;
 temp := 3600 * (9216000 / Resol_AD) * frac(temp / 25600);
 PEC_Step_Codeur_Arcsec := PEC_Step_Codeur_Arcsec + temp;
 result := PEC_Step_Codeur_Arcsec;
 PEC_Step_Old := PEC_Step;
End;

Function GetRawPositionDelta_arcsecCodeurs:Double;stdcall;
Begin
 result:=0;
End;

Function GetRAWAlphaDelta(Var CountsRA,CountsDEC:Integer):Boolean;
Begin
 (*
   This function aims at providing the raw counts of the encoders
   if Encoders systems supports
 *)
 CountsRA  :=AlphaRaw;
 CountsDEC :=DeltaRaw;
 result:=NOError;
end;

//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
// POINTAGE AUTO

Function InitTelescopeSeeker_DLL(Var Alpha,Delta,Latitude,Longitude,Accuracy:Double):Boolean;  stdcall;

Var
  MoveStepsAlpha,MoveStepsDelta : integer;
  Alphatemp,Deltatemp: double;
  MMSB,MSB,LSB,LLSB: byte;
  Err_COM:Boolean;
  astring: shortstring;

const   TABLE_ACC: array[0..1999] of integer =  (60000,24853,19070,16077,14164,12805,11776,10961,10294,9737,9261,8849,8487,8166,7880,7621,
                                                  7386,7172,6975,6794,6626,6470,6325,6189,6061,5941,5828,5721,5620,5524,5432,5345,5263,5183,
                                                  5108,5035,4966,4899,4835,4773,4714,4657,4602,4549,4497,4448,4399,4353,4308,4264,4222,4180,
                                                  4140,4102,4064,4027,3991,3956,3922,3889,3857,3825,3795,3765,3735,3707,3679,3652,3625,3599,
                                                  3573,3548,3523,3499,3476,3453,3430,3408,3386,3365,3344,3323,3303,3283,3264,3244,3226,3207,
                                                  3189,3171,3154,3136,3119,3103,3086,3070,3054,3038,3023,3008,2993,2978,2963,2949,2935,2921,
                                                  2907,2893,2880,2867,2854,2841,2828,2816,2804,2791,2779,2768,2756,2744,2733,2722,2711,2700,
                                                  2689,2678,2667,2657,2646,2636,2626,2616,2606,2596,2587,2577,2568,2558,2549,2540,2531,2522,
                                                  2513,2504,2496,2487,2479,2470,2462,2454,2445,2437,2429,2421,2414,2406,2398,2390,2383,2375,
                                                  2368,2361,2353,2346,2339,2332,2325,2318,2311,2304,2298,2291,2284,2278,2271,2265,2258,2252,
                                                  2245,2239,2233,2227,2221,2215,2209,2203,2197,2191,2185,2179,2174,2168,2162,2157,2151,2146,
                                                  2140,2135,2129,2124,2119,2113,2108,2103,2098,2093,2088,2083,2078,2073,2068,2063,2058,2053,
                                                  2048,2044,2039,2034,2030,2025,2020,2016,2011,2007,2002,1998,1993,1989,1985,1980,1976,1972,
                                                  1967,1963,1959,1955,1951,1947,1943,1939,1934,1930,1926,1923,1919,1915,1911,1907,1903,1899,
                                                  1895,1892,1888,1884,1881,1877,1873,1870,1866,1862,1859,1855,1852,1848,1845,1841,1838,1834,
                                                  1831,1827,1824,1821,1817,1814,1811,1807,1804,1801,1798,1794,1791,1788,1785,1782,1779,1775,
                                                  1772,1769,1766,1763,1760,1757,1754,1751,1748,1745,1742,1739,1736,1733,1731,1728,1725,1722,
                                                  1719,1716,1714,1711,1708,1705,1703,1700,1697,1694,1692,1689,1686,1684,1681,1678,1676,1673,
                                                  1671,1668,1665,1663,1660,1658,1655,1653,1650,1648,1645,1643,1640,1638,1635,1633,1631,1628,
                                                  1626,1623,1621,1619,1616,1614,1612,1609,1607,1605,1602,1600,1598,1596,1593,1591,1589,1587,
                                                  1584,1582,1580,1578,1576,1574,1571,1569,1567,1565,1563,1561,1559,1556,1554,1552,1550,1548,
                                                  1546,1544,1542,1540,1538,1536,1534,1532,1530,1528,1526,1524,1522,1520,1518,1516,1514,1512,
                                                  1510,1509,1507,1505,1503,1501,1499,1497,1495,1493,1492,1490,1488,1486,1484,1482,1481,1479,
                                                  1477,1475,1474,1472,1470,1468,1466,1465,1463,1461,1460,1458,1456,1454,1453,1451,1449,1448,
                                                  1446,1444,1443,1441,1439,1438,1436,1434,1433,1431,1429,1428,1426,1425,1423,1421,1420,1418,
                                                  1417,1415,1413,1412,1410,1409,1407,1406,1404,1403,1401,1400,1398,1396,1395,1393,1392,1390,
                                                  1389,1387,1386,1385,1383,1382,1380,1379,1377,1376,1374,1373,1371,1370,1369,1367,1366,1364,
                                                  1363,1362,1360,1359,1357,1356,1355,1353,1352,1350,1349,1348,1346,1345,1344,1342,1341,1340,
                                                  1338,1337,1336,1334,1333,1332,1330,1329,1328,1326,1325,1324,1323,1321,1320,1319,1317,1316,
                                                  1315,1314,1312,1311,1310,1309,1307,1306,1305,1304,1303,1301,1300,1299,1298,1296,1295,1294,
                                                  1293,1292,1290,1289,1288,1287,1286,1284,1283,1282,1281,1280,1279,1277,1276,1275,1274,1273,
                                                  1272,1271,1269,1268,1267,1266,1265,1264,1263,1262,1260,1259,1258,1257,1256,1255,1254,1253,
                                                  1252,1251,1249,1248,1247,1246,1245,1244,1243,1242,1241,1240,1239,1238,1237,1236,1235,1234,
                                                  1232,1231,1230,1229,1228,1227,1226,1225,1224,1223,1222,1221,1220,1219,1218,1217,1216,1215,
                                                  1214,1213,1212,1211,1210,1209,1208,1207,1206,1205,1204,1203,1202,1201,1200,1200,1199,1198,
                                                  1197,1196,1195,1194,1193,1192,1191,1190,1189,1188,1187,1186,1185,1184,1184,1183,1182,1181,
                                                  1180,1179,1178,1177,1176,1175,1174,1174,1173,1172,1171,1170,1169,1168,1167,1166,1166,1165,
                                                  1164,1163,1162,1161,1160,1159,1159,1158,1157,1156,1155,1154,1153,1153,1152,1151,1150,1149,
                                                  1148,1147,1147,1146,1145,1144,1143,1142,1142,1141,1140,1139,1138,1138,1137,1136,1135,1134,
                                                  1133,1133,1132,1131,1130,1129,1129,1128,1127,1126,1125,1125,1124,1123,1122,1122,1121,1120,
                                                  1119,1118,1118,1117,1116,1115,1115,1114,1113,1112,1111,1111,1110,1109,1108,1108,1107,1106,
                                                  1105,1105,1104,1103,1102,1102,1101,1100,1099,1099,1098,1097,1097,1096,1095,1094,1094,1093,
                                                  1092,1091,1091,1090,1089,1089,1088,1087,1086,1086,1085,1084,1084,1083,1082,1081,1081,1080,
                                                  1079,1079,1078,1077,1077,1076,1075,1075,1074,1073,1072,1072,1071,1070,1070,1069,1068,1068,
                                                  1067,1066,1066,1065,1064,1064,1063,1062,1062,1061,1060,1060,1059,1058,1058,1057,1056,1056,
                                                  1055,1054,1054,1053,1052,1052,1051,1051,1050,1049,1049,1048,1047,1047,1046,1045,1045,1044,
                                                  1044,1043,1042,1042,1041,1040,1040,1039,1039,1038,1037,1037,1036,1035,1035,1034,1034,1033,
                                                  1032,1032,1031,1031,1030,1029,1029,1028,1027,1027,1026,1026,1025,1024,1024,1023,1023,1022,
                                                  1022,1021,1020,1020,1019,1019,1018,1017,1017,1016,1016,1015,1014,1014,1013,1013,1012,1012,
                                                  1011,1010,1010,1009,1009,1008,1008,1007,1006,1006,1005,1005,1004,1004,1003,1003,1002,1001,
                                                  1001,1000,1000,999,999,998,998,997,996,996,995,995,994,994,993,993,992,991,991,990,990,989,
                                                  989,988,988,987,987,986,986,985,985,984,983,983,982,982,981,981,980,980,979,979,978,978,977,
                                                  977,976,976,975,975,974,974,973,973,972,972,971,971,970,970,969,968,968,967,967,966,966,965,
                                                  965,964,964,963,963,962,962,962,961,961,960,960,959,959,958,958,957,957,956,956,955,955,954,
                                                  954,953,953,952,952,951,951,950,950,949,949,948,948,947,947,947,946,946,945,945,944,944,943,
                                                  943,942,942,941,941,940,940,940,939,939,938,938,937,937,936,936,935,935,935,934,934,933,933,
                                                  932,932,931,931,930,930,930,929,929,928,928,927,927,926,926,926,925,925,924,924,923,923,923,
                                                  922,922,921,921,920,920,919,919,919,918,918,917,917,916,916,916,915,915,914,914,914,913,913,
                                                  912,912,911,911,911,910,910,909,909,908,908,908,907,907,906,906,906,905,905,904,904,904,903,
                                                  903,902,902,901,901,901,900,900,899,899,899,898,898,897,897,897,896,896,895,895,895,894,894,
                                                  893,893,893,892,892,891,891,891,890,890,889,889,889,888,888,888,887,887,886,886,886,885,885,
                                                  884,884,884,883,883,883,882,882,881,881,881,880,880,880,879,879,878,878,878,877,877,876,876,
                                                  876,875,875,875,874,874,874,873,873,872,872,872,871,871,871,870,870,869,869,869,868,868,868,
                                                  867,867,867,866,866,865,865,865,864,864,864,863,863,863,862,862,862,861,861,860,860,860,859,
                                                  859,859,858,858,858,857,857,857,856,856,856,855,855,855,854,854,853,853,853,852,852,852,851,
                                                  851,851,850,850,850,849,849,849,848,848,848,847,847,847,846,846,846,845,845,845,844,844,844,
                                                  843,843,843,842,842,842,841,841,841,840,840,840,839,839,839,838,838,838,837,837,837,836,836,
                                                  836,835,835,835,834,834,834,833,833,833,833,832,832,832,831,831,831,830,830,830,829,829,829,
                                                  828,828,828,827,827,827,827,826,826,826,825,825,825,824,824,824,823,823,823,822,822,822,822,
                                                  821,821,821,820,820,820,819,819,819,818,818,818,818,817,817,817,816,816,816,815,815,815,815,
                                                  814,814,814,813,813,813,812,812,812,812,811,811,811,810,810,810,809,809,809,809,808,808,808,
                                                  807,807,807,807,806,806,806,805,805,805,805,804,804,804,803,803,803,803,802,802,802,801,801,
                                                  801,800,800,800,800,799,799,799,799,798,798,798,797,797,797,797,796,796,796,795,795,795,795,
                                                  794,794,794,793,793,793,793,792,792,792,792,791,791,791,790,790,790,790,789,789,789,789,788,
                                                  788,788,787,787,787,787,786,786,786,786,785,785,785,784,784,784,784,783,783,783,783,782,782,
                                                  782,782,781,781,781,780,780,780,780,779,779,779,779,778,778,778,778,777,777,777,777,776,776,
                                                  776,776,775,775,775,774,774,774,774,773,773,773,773,772,772,772,772,771,771,771,771,770,770,
                                                  770,770,769,769,769,769,768,768,768,768,767,767,767,767,766,766,766,766,765,765,765,765,764,
                                                  764,764,764,763,763,763,763,762,762,762,762,761,761,761,761,760,760,760,760,759,759,759,759,
                                                  758,758,758,758,757,757,757,757,757,756,756,756,756,755,755,755,755,754,754,754,754,753,753,
                                                  753,753,752,752,752,752,752,751,751,751,751,750,750,750,750,749,749,749,749,748,748,748,748,
                                                  748,747,747,747,747,746,746,746,746,745,745,745,745,745,744,744,744,744,743,743,743,743,742,
                                                  742,742,742,742,741,741,741,741,740,740,740,740,740,739,739,739,739,738,738,738,738,738,737,
                                                  737,737,737,736,736,736,736,736,735,735,735,735,734,734,734,734,734,733,733,733,733,732,732,
                                                  732,732,732,731,731,731,731,731,730,730,730,730,729,729,729,729,729,728,728,728,728,727,727,
                                                  727,727,727,726,726,726,726,726,725,725,725,725,725,724,724,724,724,723,723,723,723,723,722,
                                                  722,722,722,722,721,721,721,721,721,720,720,720,720,720,719,719,719,719,718,718,718,718,718,
                                                  717,717,717,717,717,716,716,716,716,716,715,715,715,715,715,714,714,714,714,714,713,713,713,
                                                  713,713,712,712,712,712,712,711,711,711,711,711,710,710,710,710,710,709,709,709,709,709,708,
                                                  708,708,708,708,707,707,707,707,707,706,706,706,706,706,705,705,705,705,705,704,704,704,704,
                                                  704,704,703,703,703,703,703,702,702,702,702,702,701,701,701,701,701,700,700,700,700,700,699,
                                                  699,699,699,699,699,698,698,698,698,698,697,697,697,697,697,696,696,696,696,696,696,695,695,
                                                  695,695,695,694,694,694,694,694,693,693,693,693,693,693,692,692,692,692,692,691,691,691,691,
                                                  691,691,690,690,690,690,690,689,689,689,689,689,689,688,688,688,688,688,687,687,687,687,687,
                                                  687,686,686,686,686,686,685,685,685,685,685,685,684,684,684,684,684,683,683,683,683,683,683,
                                                  682,682,682,682,682,682,681,681,681,681,681,681,680,680,680,680,680,679,679,679,679,679,679,
                                                  678,678,678,678,678,678,677,677,677,677,677,677,676,676,676,676,676,675,675,675,675,675,675,
                                                  674,674,674,674,674,674,673,673,673,673,673,673,672,672,672,672,672,672,671,671,671,671);

function calc_time_seeker(numstep:integer):double;                 // Evaluation du temps de deplacement en AD pour correction en temps sidrale

var     DELAY,m,y0,x0,i,index,resol,DELAY_TARGET:integer;

begin
 i := 0;
 index := 0;
 resol := 64;

 DELAY_TARGET := 64 * round(625000 / V_Point_R_AD);

 DELAY := TABLE_ACC[index]; 		                                // Delai valu selon la table d'acclration
						                        // bas sur A[i] = (i^0.5 - (i - 1)^0.5)A[0]
 m := TABLE_ACC[1899] - TABLE_ACC[1999];
 m := 100 div m;
 y0 := TABLE_ACC[1999] + 2000 div m;
 x0 := y0 * m;

 if numstep < 0 then numstep := -numstep;


	//	Dclanche le mouvement rapide avec une rsolution 1/2 pas

 while numstep > resol do			                        // Boucle de positionnement
  begin

   numstep := numstep - resol;

   if ((i < numstep) and (DELAY > DELAY_TARGET)) then
    begin
     i := i + resol;			// Conditions pour changer le dlai de gnration des pas
     index := index + 1;
     index := index + Accel_AD;
     if (index < 2000) then DELAY := DELAY + TABLE_ACC[index] 		// Delai1 valu selon la table d'acclration

									// bas sur A[i] = (i^0.5 - (i - 1)^0.5)A[0]
     else
      if (index < x0) then DELAY := DELAY + (y0 - index div m);         // Delai2 valu selon une droite partie suprieure de la courbe d'acclration
    end
    else
     if (i > numstep) then
      begin
       i := i - resol;
       index := index - 1;
       index := index - Accel_AD;
       if (index < 2000)then DELAY := DELAY + TABLE_ACC[index] 		// Delai valu selon la table d'acclration
									// bas sur A[i] = (i^0.5 - (i - 1)^0.5)A[0]
       else
        if (index < x0) then DELAY := DELAY + (y0 - index div m);       // Delai2 valu selon une droite partie suprieure de la courbe d'acclration
      end;
  end;

  //ShowMessage('DELAY : '+inttostr(DELAY));

  if (numstep > 0) then
  begin

   while(numstep > 0) do
    begin
     numstep := numstep - 1;
     if (V_Corr_P_AD) > 0 then
      DELAY := DELAY + round(int(625000 / V_Corr_P_AD));
    end;
  end;
 calc_time_seeker := DELAY / 625000 ;
end;


begin
 // Alpha     : RA Distination coordinates
 // Delta     : DEC Distination coordinates
 // Longitude : observation Longitude place coordinates
 // Latitude  : observation Latitude place coordinates
 // Accuracy  : Desired pointing accuracy
 //

// while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
 AccesCom(NumCOM);

 AlphaTarget := Alpha;
 DeltaTarget := Delta;

 Deltatemp := Delta - GDelta;

 MoveStepsDelta := round(Resol_DEC * DeltaTemp / (2*Pi));

 //ShowMessage('MoveStepsDelta : '+inttostr(MoveStepsDelta));

 AlphaTemp := GAlpha;

 if Alpha - Pi > GAlpha then
  AlphaTemp := AlphaTemp + 2*Pi
 else
  if GAlpha - Pi > Alpha then
   Alpha := Alpha + 2*Pi;

 AlphaTemp := AlphaTemp - Alpha;

 MoveStepsAlpha := round(Resol_AD * AlphaTemp / (2*Pi));

 try
//   Open_Cod_Eng(Err_COM);
   try
     //Alpha et Delta sont en RADIANS !!!

     if (abs(MoveStepsDelta) > 0) then
      begin
        MMSB:= (abs(MoveStepsDelta) SHR 24) and $7F;
        MSB:= (abs(MoveStepsDelta) SHR 16) and $FF;
        LSB:= (abs(MoveStepsDelta) SHR 8) and $FF;
        LLSB:= abs(MoveStepsDelta) and $FF;
        if MoveStepsDelta < 0 then MMSB := MMSB or $80;

        while deviceready(1) <> 1 do ; //Application.ProcessMessages;
        astring:='p'+chr(MMSB)+chr(MSB)+chr(LSB)+chr(LLSB);
        sendcommand(astring,0,1);
      end;

     if (abs(MoveStepsAlpha) > 0) then
      begin

        //MoveStepsAlpha := MoveStepsAlpha + round(calc_time_seeker(MoveStepsAlpha) * V_Guidage_AD);

        MMSB:= (abs(MoveStepsAlpha) SHR 24) and $7F;
        MSB:= (abs(MoveStepsAlpha) SHR 16) and $FF;
        LSB:= (abs(MoveStepsAlpha) SHR 8) and $FF;
        LLSB:= abs(MoveStepsAlpha) and $FF;
        if MoveStepsAlpha < 0 then MMSB := MMSB or $80;

        //showmessage('Temps: '+floattostr(calc_time_seeker(MoveStepsAlpha)));

        while deviceready(0) <> 1 do ; //Application.ProcessMessages;
        astring:='p'+chr(MMSB)+chr(MSB)+chr(LSB)+chr(LLSB);
        sendcommand(astring,0,0);
      end;

   finally
//     Closehandle(hCom);
   end;
 finally
   AccesCom(NumCOM);
 end;

 AlphaOldPoint := Alpha;
 DeltaOldPoint := Delta;

 // Must be send back
Accuracy:=DegToRad(1/30);
result:=true;
end;


// Here proceeds actions according to latest telescope position.
// This is called by a separed PROCESS/THREAD, so be very carefull here, do not DISPLAY
// any messages, or don't be stuck here, otherwise he could make crash PRISM
// Don't take care of pointing accuracy here, because PRISM does it for you and
// stops the PROCESS/THREAD if necessary.
// Sometimes, nothing has to be done here, because a simple order to the device can be
// provided thru InitTelescopeSeeker_DLL, so leave it like that. (ie lx200 case)

Function Execute_TelescopeSeeker_DLL(var Alpha,Delta:Double):Boolean;  stdcall;
begin
        result:= NOERROR;
end;

// Executes here code to stop forcefully the telescope slewing...
// Do not put graphic stuff here... this things goes into a thread.

Function StopSlewing_TelescopeSeekerDevice_DLL:Boolean;  stdcall;

Var
   Err_COM:Boolean;

begin
//  while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
    AccesCom(NumCOM);
  try
//    Open_Cod_Eng(Err_COM);
    try
      StopSlewing(0);
      StopSlewing(1);
    finally
//      CloseHandle(hCom);
    end;
  finally
    AccesCom(NumCOM);
  end;
  result:=true;
end;



// Close what is necessary to be closed after pointing completion
// Sometimes, nothing has to be closed, so leave it like that.

Function CloseTelescopeSeeker_DLL:Boolean;   stdcall;
begin
 result:=NOERROR;
end;


//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
// Telescope drives


Type

 TEngineStruct=Record
                 AbleTo_WidthStand_SpeedChanges:Boolean;  // Is drives able to change speed upon demand ?
                 AbleTo_ApplyOffsets           :Boolean;  // Engine able to achieve accurate offsets
                 AbleTo_Stop_All               :Boolean;  // Can all drives be ALL stopped
                 Able_ToSupportEmbeddedSpeeds  :Boolean; // Can the drives has solar,Mpec and moon speeds embedded
                 RateEvent                     :Integer; // Check every RateEvent Ms if telescope has been moved
                 HasAConsole                   :Boolean; // Has a console....
                 HasASpeedpanel                :Boolean; // Has a speed panel....
                 AbleToPulseGuide              :Boolean; // Able to perform guiding
               end;


 TDirection = (DNord,DSud,DEst,DOuest,DNone);
 TSpeed     = (SMax,SMedium,SSideral,SNone );

Var
  SpeedAlpha:TSpeed;   // Pour connatre la vitesse impose en Alpha suivant l'objet TSpeed
  SpeedDelta:TSpeed;   // Pour connatre la vitesse impose en Delta suivant l'objet TSpeed

//Fait
Function InitTelescopeEngine_DLL(Var EngineStructConf:TEngineStruct):Boolean; stdcall;

Var
   Err_COM:Boolean;

Begin
 //ShowMessage('InitTelescopeEngine!');
  With EngineStructConf do
   begin
     AbleTo_WidthStand_SpeedChanges:=True;   // If True
                                              // ReturnToSideralSpeed must be filled with appropriate code
                                              // SetDriveSpeed must be filled with appropriate code


     AbleTo_ApplyOffsets:=True; // If True
                                // MoveOffsetDirectAD must be filled with appropriate code

     AbleTo_Stop_All:=True; // If True
                            // STOP_ALL_Telescope must be filled with appropriate code

     Able_ToSupportEmbeddedSpeeds:=True; // Can the drives has solar,Mpec and moon speeds embedded
                                         // If True
                                         // ReturnToSideralSpeed
                                         // StartPecEngine            name 'StartPecSpeedEngineDevice_DLL',
                                         // StartSolarEngine          name 'StartSolarSpeedEngineDevice_DLL',
                                         // StartMoonEngine           name 'StartMoonSpeedEngineDevice_DLL',
                                         // Must be filled with appropriate code

     HasAConsole                 :=True; // Must be able to show a Console panel...
     HasASpeedpanel              :=True; // Must be able to show a Speed panel...

     RateEvent:=10000;

  end;

  while not AccesComAllowance(NumCOM) do;
  AccesCom(NumCOM);
  try
//   Open_Cod_Eng(Err_COM);
   try
     //test la prsence du module.

   finally
//     CloseHandle(hCom);
   end;
  finally
    AccesCom(NumCOM);
  end;
  result:=true;
end;

// If HasAConsole=False this function does not need to exist
Function  ShowConsoleEngine(Aowner:Tcomponent):Boolean; stdcall;
Begin
 FormConsole:=TFormConsole.create(Aowner);  // To pass Aowner is VERY important
 FormConsole.PageConsole.ActivePageIndex := 0;
 FormConsole.Show;
 result:=NOError;
end;

// If HasASpeedpanel=False this function does not need to exist
Function  ShowSpeedEngine(Aowner:Tcomponent):Boolean; stdcall;
begin
  //SpeedEngine:=TSpeedEngine.create(Aowner);  // To pass Aowner is VERY important
  //SpeedEngine.Show;
  result:=NOERROR;
end;


Function GetSpeedAlpha(Speed:TSpeed;Var RelativeSpeed:Integer):Boolean; stdcall;
Begin
 //ShowMessage('GetSpeedAlpha!');
 // Ce sont les vitesses de centrage qui sont reprsentes ici, c'est--dire celles
 // qui servent dans la raquette du tlescope

 case Speed of             // With respect to the sideral speed (ie = 1x);
        SMax:   RelativeSpeed := round(V_Point_R_AD / V_Guidage_AD);
       //Rcuprer les vitesses par rapport  la vitesse sidrale ( voir l'unit Vitesses.pas ) :
       // Comme RelativeSpeed doit necessairement tre entier, on est oblig de la considrer
       // gal  1 pour des vitesses rellement infrieure  1 fois la vitesse sidrale :
        SMedium:  RelativeSpeed := round(V_Point_L_AD / V_Guidage_AD);
          // idem mais en fonction de la vitesse de centrage mdium

        SSideral:  RelativeSpeed := round(V_Corr_P_AD / V_Guidage_AD);
           // idem mais en fonction de la vitesse de centrage minimum
        SNone:  RelativeSpeed:=0;
 end;
 result:=NOERROR;
end;

//Fait
Function GetSpeedDelta(Speed:TSpeed;Var RelativeSpeed:Integer):Boolean; stdcall;
Begin
 case Speed of             // With respect to the sideral speed (ie = 1x);
                SMax: RelativeSpeed := round(V_Point_R_DEC / V_Corr_P_DEC);
                SMedium: RelativeSpeed := round(V_Point_L_DEC / V_Corr_P_DEC);
                SSideral: RelativeSpeed:=1;
                SNone:  RelativeSpeed:=0;
 end;
 result:=NOERROR;
end;


// Stops the telescope (Alpha;delta : axis) and put it back in tracking mode
Function STOPTelescope(Alpha,Delta:Boolean):Boolean; stdcall;
// Stoppe le tlescope depuis les instructions envoyes par la raquette du tlescope
// Cette procdure est utile aprs avoir compris "MoveTelescope" ...
Var
   Err_COM:Boolean;
   astring: shortstring;

Begin
  while not AccesComAllowance(NumCOM) do ; //Application.ProcessMessages;
    AccesCom(NumCOM);
    try
//      Open_Cod_Eng(Err_COM);
      try
        if Alpha then     // Si l'arrt en Alpha a t demand ...
          begin
                case SpeedAlpha of             // With respect to the sideral speed (ie = 1x);
                        SMax: StopSlewing(0);
                        SMedium: StopSlewing(0);
                        SSideral: begin
                                        while not deviceready(0) = 1 do ; //Application.ProcessMessages;
                                        astring:='S'+chr(0)+chr(0)+chr(0)+chr(0);
                                        sendcommand(astring,0,0);
                                  end;
                        SNone:  ;
                end;
          end;

        if Delta then    // Si l'arrt en Delta a t demand
          begin
                case SpeedDelta of             // With respect to the sideral speed (ie = 1x);
                        SMax: StopSlewing(1);
                        SMedium: StopSlewing(1);
                        SSideral: begin
                                        while not deviceready(1) = 1 do ; //Application.ProcessMessages;
                                        astring:='S'+chr(0)+chr(0)+chr(0)+chr(0);
                                        sendcommand(astring,0,1);
                                  end;
                        SNone:  ;
                end;
          end;
      finally
//        CloseHandle(hCom);
      end;
    finally
      AccesCom(NumCOM);
    end;
  result:=true;
end;

Function MoveTelescope(Direction:TDirection;Speed:TSpeed):Boolean; stdcall;
Var
   Err_COM:Boolean;
   astring: shortstring;

Begin
// while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
  AccesCom(NumCOM);
  try
//   Open_Cod_Eng(Err_COM);
   try
    FlushFileBuffers(hCom);
    case Direction of
      DNord:begin
            // On a demand d'aller vers le Nord : Delta augmente
                case Speed of
                        SMax:   begin   //Fournir la vitesse VMax :
                                 while not deviceready(1) = 1 do ; //Application.ProcessMessages;
                                 astring:='X'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,1);
                                 SpeedDelta := SMax;
                                end;
                        SMedium:begin        //Fournir la vitesse VMed :
                                 while not deviceready(1) = 1 do ; //Application.ProcessMessages;
                                 astring:='G'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,1);
                                 SpeedDelta := SMedium;
                                end;
                        SSideral:begin     // Fournir la vitesse VMin :
                                 while not deviceready(1) = 1 do ; // Application.ProcessMessages;
                                 astring:='D'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,1);
                                 SpeedDelta := SSideral;
                                end;
                end;
            end;

      DSud :begin
            // On a demand d'aller vers le Sud : Delta diminue
                case Speed of
                        SMax:   begin   //Fournir la vitesse VMax :
                                 while not deviceready(1) = 1 do ; //Application.ProcessMessages;
                                 astring:='W'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,1);
                                 SpeedDelta := SMax;
                                end;
                        SMedium:begin        //Fournir la vitesse VMed :
                                 while not deviceready(1) = 1 do ; //Application.ProcessMessages;
                                 astring:='F'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,1);
                                 SpeedDelta := SMedium;
                                end;
                        SSideral:begin     // Fournir la vitesse VMin :
                                 while not deviceready(1) = 1 do ; //Application.ProcessMessages;
                                 astring:='Q'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,1);
                                 SpeedDelta := SSideral;
                                end;
                end;
            end;

      DEst :begin
            // On a demand d'aller vers l'Est : Alpha augmente
                case Speed of
                        SMax:   begin   //Fournir la vitesse VMax :
                                 while not deviceready(0) = 1 do ; //Application.ProcessMessages;
                                 astring:='W'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,0);
                                 SpeedAlpha := SMax;
                                end;
                        SMedium:begin        //Fournir la vitesse VMed :
                                 while not deviceready(0) = 1 do ; //Application.ProcessMessages;
                                 astring:='F'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,0);
                                 SpeedAlpha := SMedium;
                                end;
                        SSideral:begin     // Fournir la vitesse VMin :
                                 while not deviceready(0) = 1 do ; //Application.ProcessMessages;
                                 astring:='Q'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,0);
                                 SpeedAlpha := SSideral;
                                end;
                end;
            end;

      DOuest:begin
             // On a demand d'aller vers l'Ouest : Alpha diminue
                case Speed of
                        SMax:   begin   //Fournir la vitesse VMax :
                                 while not deviceready(0) = 1 do ; //Application.ProcessMessages;
                                 astring:='X'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,0);
                                 SpeedAlpha := SMax;
                                end;
                        SMedium:begin        //Fournir la vitesse VMed :
                                 while not deviceready(0) = 1 do ; //Application.ProcessMessages;
                                 astring:='G'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,0);
                                 SpeedAlpha := SMedium;
                                end;
                        SSideral:begin     // Fournir la vitesse VMin :
                                 while not deviceready(0) = 1 do ; //Application.ProcessMessages;
                                 astring:='D'+chr(0)+chr(0)+chr(0)+chr(0);
                                 sendcommand(astring,0,0);
                                 SpeedAlpha := SSideral;
                                end;
                end;
             end;
    end;
    finally
//      Closehandle(hCom);
    end;
  finally
    AccesCom(NumCOM);
  end;
 result:=NOERROR;
end;


// This function is called by a timer every RateEvent ms, this timer
// has his own PROCESS/THREAD so do not display any form here
// Can be used to check to some extent the telescope status
// If not used leave result:=NOError;

Function MoveEventTelescope:Boolean; stdcall;
Begin
  result:=NOERROR;
end;

// Stops completely the telescope
// If AbleTo_Stop_All is false, leave it as it is.
Function STOP_ALL_Telescope:Boolean; stdcall;
Var
   Err_COM:Boolean;
   astring:ShortString;

begin
//  while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
    AccesCom(NumCOM);
  try
//    Open_Cod_Eng(Err_COM);
    try
      //StopSlewing(0);
      //StopSlewing(1);
      while deviceready(0) <> 1 do ; //Application.ProcessMessages;
      astring:='P'+chr(0)+chr(0)+chr(0)+chr(0);                      // Commande Park pour AD
      sendcommand(astring,0,0);
      while deviceready(1) <> 1 do ; //Application.ProcessMessages;
      astring:='R'+chr(0)+chr(0)+chr(0)+chr(0);                      // Commande vitesse = 0 en DEC;
      sendcommand(astring,0,1);
    finally
//      CloseHandle(hCom);
    end;
  finally
    AccesCom(NumCOM);
  end;
  result:=true;
end;


// Applies an offset to the telescope
// If AbleTo_ApplyOffsets is false, leave it as it is.

Function MoveOffsetDirectAD(AlphaD,DeltaD:Double;Speed:TSpeed):Boolean; stdcall;
// Cette fonction est quasiment en tout point identique  "InitTelescopeSeeker_DLL".
// Pour les commentaires voir cette fonction.
// En fait, on rajoute le dcalage  GAlpha et  GDelta et on pointe vers ce qu'on n'a rajout ...
Var
   NumOfSteps : longint;
   astring:ShortString;
   MMSB,MSB,LSB,LLSB: byte;
   Err_COM:Boolean;

Begin
// while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
  AccesCom(NumCOM);
  try
//   Open_Cod_Eng(Err_COM);
   try
     NumOfSteps := round(Resol_AD * AlphaD / (2*Pi));
     if abs(NumOfSteps) > 0 then
      begin
        MMSB:= (abs(NumOfSteps) SHR 24) and $FF;
        MSB:= (abs(NumOfSteps) SHR 16) and $FF;
        LSB:= (abs(NumOfSteps) SHR 8) and $FF;
        LLSB:= abs(NumOfSteps) and $FF;
        if NumOfSteps < 0 then MMSB := MMSB or $80;

        while deviceready(0) <> 1 do ; //Application.ProcessMessages;
        astring:='p'+chr(MMSB)+chr(MSB)+chr(LSB)+chr(LLSB);
        sendcommand(astring,0,0);
      end;

     NumOfSteps := round(Resol_DEC * DeltaD / (2*Pi));
     if abs(NumOfSteps) > 0 then
      begin
        MMSB:= (abs(NumOfSteps) SHR 24) and $FF;
        MSB:= (abs(NumOfSteps) SHR 16) and $FF;
        LSB:= (abs(NumOfSteps) SHR 8) and $FF;
        LLSB:= abs(NumOfSteps) and $FF;
        if NumOfSteps < 0 then MMSB := MMSB or $80;

        while deviceready(1) <> 1 do ; //Application.ProcessMessages;
        astring:='p'+chr(MMSB)+chr(MSB)+chr(LSB)+chr(LLSB);
        sendcommand(astring,0,1);
      end;
   finally
//    Closehandle(hCom);
   end;
  finally
    AccesCom(NumCOM);
  end;
 result:=NOERROR;
end;

// If AbleTo_WidthStand_SpeedChanges is false, leave it as it is.
// Otherwise allows the telescope to come back to sideral speed
// If PEC correction is activated, here you should use this function to desactivate it
// as so as the moon and solar speeds.
Function ReturnToSideralSpeed:Boolean; stdcall;
Var
   Err_COM:Boolean;
   astring: shortstring;

Begin
//  while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
    AccesCom(NumCOM);
    try
//      Open_Cod_Eng(Err_COM);
      try
        while not deviceready(0) = 1 do ; //Application.ProcessMessages;
        astring:='N'+chr(0)+chr(0)+chr(0)+chr(0);                      // Commande No Park
        sendcommand(astring,0,0);
        while not deviceready(0) = 1 do ; //Application.ProcessMessages;
        astring:='S'+chr(0)+chr(0)+chr(0)+chr(0);                      // Vitesse Siderale en AD
        sendcommand(astring,0,0);
        while not deviceready(1) = 1 do ; //Application.ProcessMessages;
        astring:='S'+chr(0)+chr(0)+chr(0)+chr(0);
        sendcommand(astring,0,1);
        V_Guidage_AD := V_Guidage_AD_SID;
      finally
//        CloseHandle(hCom);
      end;
    finally
      AccesCom(NumCOM);
    end;
  result:=true;
end;

// If AbleTo_WidthStand_SpeedChanges is false, leave it as it is.
// Otherwise allows the telescope to come back to sideral speed
// If PEC correction is activated, here you should use this function to desactivate it
// as so as the moon and solar speeds.
Function StopFreeSpeed:Boolean; stdcall;
Var
   Err_COM:Boolean;
   astring: shortstring;

Begin
  while not AccesComAllowance(NumCOM) do ; //Application.ProcessMessages;
    AccesCom(NumCOM);
    try
//      Open_Cod_Eng(Err_COM);
      try
        while not deviceready(0) = 1 do ; //Application.ProcessMessages;
        astring:='N'+chr(0)+chr(0)+chr(0)+chr(0);                      // Commande No Park
        sendcommand(astring,0,0);
        while not deviceready(0) = 1 do ; //Application.ProcessMessages;
        astring:='S'+chr(0)+chr(0)+chr(0)+chr(0);
        sendcommand(astring,0,0);
        while not deviceready(1) = 1 do ; //Application.ProcessMessages;
        astring:='S'+chr(0)+chr(0)+chr(0)+chr(0);
        sendcommand(astring,0,1);
        V_Guidage_AD := V_Guidage_AD_SID;
      finally
//        CloseHandle(hCom);
      end;
    finally
      AccesCom(NumCOM);
    end;
  result:=true;
end;

// If AbleTo_WidthStand_SpeedChanges is false, leave it as it is.
// en rad/heures et pour un objet mobile, type comete ou aster
Function SetFreeSpeed(AlphaSpeed_rad_H,DeltaSpeed_rad_H:Double):Boolean;
Var
   Err_COM:Boolean;
   astring:ShortString;
   SIGNE,MSB,LSB,LLSB: byte;
   aval: Double;

begin
 //showmessage('OK_debut');
//  while not AccesComAllowance(NumCOM) do ;//Application.ProcessMessages;
    AccesCom(NumCOM);
  try
//    Open_Cod_Eng(Err_COM);
    try
      begin
        aval := AlphaSpeed_rad_H * Resol_AD / (2 * 3600 * PI);
        aval := 625000 / aval;
        V_Guidage_AD := aval;
        MSB:= (round(abs(aval)) SHR 8) and $FF;
        LSB:= (round(abs(aval))) and $FF;
        LLSB:= round(abs(10 * frac(aval))) and $FF;
        SIGNE := 0;
        if aval > 0 then SIGNE := 1;
        while deviceready(0) <> 1 do ; //Application.ProcessMessages;
        astring:='R'+chr(MSB)+chr(LSB)+chr(LLSB)+chr(SIGNE);
        sendcommand(astring,0,0);
      end;

      begin
        aval := DeltaSpeed_rad_H * Resol_DEC / (2 * 3600 * PI);
        aval := 625000 / aval;
        MSB:= (round(abs(aval)) SHR 8) and $FF;
        LSB:= (round(abs(aval))) and $FF;
        LLSB:= round(abs(10 * frac(aval))) and $FF;
        SIGNE := 0;
        if aval > 0 then SIGNE := 1;
        while deviceready(1) <> 1 do ; //Application.ProcessMessages;
        astring:='R'+chr(MSB)+chr(LSB)+chr(LLSB)+chr(SIGNE);
        sendcommand(astring,0,1);
      end;
    finally
//      CloseHandle(hCom);
    end;
  finally
    AccesCom(NumCOM);
  end;
  result:=true;
  //result:=NOERROR;
end;

// Start Solar speed engine ... stopped by ReturnToSideralSpeed
Function StartSolarEngine:Boolean; stdcall;

Var
   Err_COM:Boolean;
   astring:ShortString;
   SIGNE,MSB,LSB,LLSB: byte;
   aval: Double;

begin
 //showmessage('OK_debut');
//  while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
    AccesCom(NumCOM);
  try
//    Open_Cod_Eng(Err_COM);
    try
      begin
        V_Guidage_AD := round(V_Guidage_AD_SID * 86164.101 / 86400);
        aval := V_Guidage_AD;
        aval := 625000 / aval;
        MSB:= (round(abs(aval)) SHR 8) and $FF;
        LSB:= (round(abs(aval))) and $FF;
        LLSB:= round(abs(10 * frac(aval))) and $FF;
        SIGNE := 0;
        if aval > 0 then SIGNE := 1;
        while deviceready(0) <> 1 do ; //Application.ProcessMessages;
        astring:='R'+chr(MSB)+chr(LSB)+chr(LLSB)+chr(SIGNE);
        sendcommand(astring,0,0);
      end;
    finally
//      CloseHandle(hCom);
    end;
  finally
    AccesCom(NumCOM);
  end;
  result:=true;
  //result:=NOERROR;
end;

// Start Moon speed engine ... stopped by ReturnToSideralSpeed
Function StartMoonEngine:Boolean; stdcall;
Var
   Err_COM:Boolean;
   astring:ShortString;
   SIGNE,MSB,LSB,LLSB: byte;
   aval: Double;

begin
 //showmessage('OK_debut');
//  while not AccesComAllowance(NumCOM) do Application.ProcessMessages;
    AccesCom(NumCOM);
  try
//    Open_Cod_Eng(Err_COM);
    try
      begin
        V_Guidage_AD := round(V_Guidage_AD_SID * 27.32166 / 27.32158);
        aval := V_Guidage_AD;
        aval := 625000 / aval;
        MSB:= (round(abs(aval)) SHR 8) and $FF;
        LSB:= (round(abs(aval))) and $FF;
        LLSB:= round(abs(10 * frac(aval))) and $FF;
        SIGNE := 0;
        if aval > 0 then SIGNE := 1;
        while deviceready(0) <> 1 do ; //Application.ProcessMessages;
        astring:='R'+chr(MSB)+chr(LSB)+chr(LLSB)+chr(SIGNE);
        sendcommand(astring,0,0);
      end;
    finally
//      CloseHandle(hCom);
    end;
  finally
    AccesCom(NumCOM);
  end;
  result:=true;
  //result:=NOERROR;
end;

// Closes the telescope engine
//Fait
Function CloseEngine:Boolean; stdcall;
begin
 //Pas la peine de la fermer puisque le port est ferme des que l'appel est termin !
 result:=NOERROR;
end;

//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
// King

Function StartKingDevice(latitude_rad,Longitude_rad,Altitude_m,pression_mb:Double):Boolean; stdcall;
begin
 King_Activated := true;
 result:=NOERROR;
end;

Function StopKingDevice:Boolean; stdcall;
begin
 King_Activated := false;
 ReturnToSideralSpeed;
 result:=NOERROR;
end;

//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
Function PulseGuide(RaInd,DecInd,MsRa,MsDec:Word):Boolean; stdcall;
// function(RaInd,DecInd,MsRa,MsDec:Word):Boolean; stdcall;
//   RaInd=0 = ALpha+
//   RaInd=1 = ALpha-
Begin
end;

Function  GetCurrentSpeedRA(Var D:Double):Boolean; stdcall;
Begin
 // Return current speed : 1.00 is sidereal speed ie 15.04 arsec/sec at delta=0
 result:=True;
 D:=1;
end;

Function  GetCurrentSpeedDEC(Var D:Double):Boolean; stdcall;
Begin
 result:=True;
 D:=1;
end;


//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
// PEC

Function StartPecSpeedEngine(Periode_arcsec,Phase_arcsec,Amplitude_arcsec:Array of Double):Boolean; stdcall;
begin
 Phase_PEC[1]:= Phase_arcsec[0];
 Phase_PEC[2]:= Phase_arcsec[1];
 Periode_PEC[1] := Periode_arcsec[0];
 Periode_PEC[2] := Periode_arcsec[1];
 Amplitude_PEC[1] := Amplitude_arcsec[0];
 Amplitude_PEC[2] := Amplitude_arcsec[1];
 //showmessage('pa= '+floattostr(Phase[1])+' Pe= '+floattostr(periode)+' A= '+floattostr(Amplitude[1]));
 PEC_Activated := true;
 FormConsole.PEC.Enabled := false;
 result:=NOERROR;
end;

Function StopPec:Boolean; // Stoppe le PEC
begin
 PEC_Activated := false;
 ReturnToSideralSpeed;
 FormConsole.PEC.Enabled := true;
 result:=NOERROR;
end;

//////// ///////// ///////// ///////// ///////// ///////// ///////// ///////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //
// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //
// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //// SETUP PANEL //

Type
 TsetupEngine_encoderEnumerator=(SetupEngine,SetupEncoders);

Procedure Init_telescope_Panel(Aowner:Tcomponent;TwinHandle:Thandle;TypeSetup:TsetupEngine_encoderEnumerator); stdcall;
//   This function has to display ITSELF a form to ask eventually about :
//   COM, Ip address, initialisation parameters, etc etc....
//   Twinhandle is PRISM's caller window handler in case of use of WIN_API procs., in order to create a modalwindow in front of all the others
//   in case of Delphi DLL please use Aowner instead.

Var SetupTelescope:TSetupTelescope;
    Form_scan_mcmt:TForm_scan_mcmt;
    DeviceIni: TIniFile;
    S: string;
    n, Len: DWORD;
    Buf: PChar;
    Value: PChar;
    found: boolean;
    c: integer;
    TmpPorts: TStringList;

begin
//  If Aowner<>Nil then  // Here it can display a setup panel given as a prototype
//  Begin
    SetupTelescope:=TSetupTelescope.create(Aowner);  // To pass Aowner is VERY important
                                                     // Otherwise the panel is not visible

    SetupTelescope.caption := 'MCMTII Telescope DLL version ';
    S := 'telescope.dll';
    n := GetFileVersionInfoSize(PChar(S), n);

    decimalseparator:='.';

    if n > 0 then
    begin
     Buf := AllocMem(n);
     GetFileVersionInfo(PChar(S), 0, n, Buf);
      if VerQueryValue(Buf, PChar('StringFileInfo\040C04E4\FileVersion'), Pointer(Value), Len) then
        SetupTelescope.caption := SetupTelescope.caption + Value;
     FreeMem(Buf, n);
    end;

  found := false;
  stopscan := false;

  DeviceIni := TIniFile.Create('mcmtii.ini');
   with DeviceIni do
   begin
    NumCOM:=ReadInteger('Configuration','Port',1);
   end;
  DeviceIni.free;

 TmpPorts := TStringList.Create;
 EnumComPorts(TmpPorts);

 SetupTelescope.ComboBoxNumCOM.Items.Assign(TmpPorts);

 try                      // on test si MCMT prsent  la mme com qu'auparavant
  Open_Cod_Eng(Err_COM);  // ouvrir le port COM
  try
   if deviceready(0) = 1 then found := true;
  finally
   CloseHandle(hCom);
  end;
  except;
 end;

 if not found then        // Sinon on scan sur les autres ports dispo
 begin
  Form_scan_mcmt:=TForm_scan_mcmt.create(Aowner);
  Form_scan_mcmt.show;
  c:=0;
  while (c < TmpPorts.Count) and (found = false)
  do
   begin
    NumCom := strtoint(TmpPorts[c]);
    Form_scan_mcmt.Panel_Info.Caption := 'Recherche MCMT sur com ' + inttostr(NumCOM);
    try
     Open_Cod_Eng(Err_COM);  // ouvrir le port COM
     try
      if deviceready(0) = 1 then
       begin
        found := true;
        SetupTelescope.ComboBoxNumCOM.ItemIndex := c;
        c:=TmpPorts.Count;
       end;
      if stopscan = true then c:=TmpPorts.Count;
     finally
      CloseHandle(hCom);
     end;
    except;
    end;
     c := c + 1;
   end;
  Form_scan_mcmt.free;
 end;

 TmpPorts.Free;

 if Not found then  //toujours pas trouv?
  begin
   if MessageDlg('Le programme n''a pas dtect MCMT!  Voulez-vous poursuivre quand mme?',
     mtConfirmation, [mbYes, mbNo], 0) = mrYes
   then
    begin
     SetupTelescope.ComboBoxNumCOM.ItemIndex:=0;
    end
    else
     begin
      SetupTelescope.Free; //Destroy...
      exit;
     end;
  end;

    DeviceIni := TIniFile.Create('mcmtii.ini');
    with DeviceIni do
    begin
     if not found then SetupTelescope.GroupBox3.visible := true else SetupTelescope.ComboBoxNumCOM.text := inttostr(NumCOM);
     GDelta:=strtofloat(ReadString('Configuration','LastDelta','0'));
     GAlpha:=strtofloat(ReadString('Configuration','LastAlpha','0'));
     SetupTelescope.AfficheNomFichier.caption := ReadString('Configuration','FirmwareFile','');
     SetupTelescope.CheckBox_DIR_AD.Checked :=ReadBool('Parameters', 'DIR_AD', False);
     SetupTelescope.CheckBox_DIR_DEC.checked:=ReadBool('Parameters', 'DIR_DEC', False);
     SetupTelescope.CheckBox_PEC.checked:=ReadBool('Parameters', 'PEC_AD', False);
     SetupTelescope.CheckBox_Raq_Can.checked:=ReadBool('Parameters', 'Raq_Can', False);
     Inv_Codeur_AD := SetupTelescope.CheckBox_DIR_AD.Checked;
     Inv_Codeur_DEC := SetupTelescope.CheckBox_DIR_DEC.checked;
     Resol_AD:= ReadInteger('Parameters', 'Resol_AD', 4608000);
     SetupTelescope.Edit_Resol_AD.text:=inttostr(Resol_AD);
     Resol_DEC:= ReadInteger('Parameters', 'Resol_DEC', 4608000);
     SetupTelescope.Edit_Resol_DEC.text:=inttostr(Resol_DEC);
     SetupTelescope.EditResolAlpha.text := ReadString('Parameters', 'ReducAlpha', '360');
     SetupTelescope.EditResolDelta.text := ReadString('Parameters', 'ReducDelta', '360');
     SetupTelescope.Edit_V_Guidage_AD.text := ReadString('Parameters', 'V_Guidage_AD', '1');
     SetupTelescope.Edit_V_Guidage_DEC.text := ReadString('Parameters', 'V_Guidage_DEC', '1');
     SetupTelescope.Edit_V_Corr_P_AD.text := ReadString('Parameters', 'V_Corr_P_AD', '1');
     SetupTelescope.Edit_V_Corr_P_DEC.text := ReadString('Parameters', 'V_Corr_P_DEC', '1');
     SetupTelescope.Edit_V_Corr_M_AD.text := ReadString('Parameters', 'V_Corr_M_AD', '1');
     SetupTelescope.Edit_V_Corr_M_DEC.text := ReadString('Parameters', 'V_Corr_M_DEC', '1');
     SetupTelescope.Edit_V_Point_L_AD.text := ReadString('Parameters', 'V_Point_L_AD', '1');
     SetupTelescope.Edit_V_Point_L_DEC.text := ReadString('Parameters', 'V_Point_L_DEC', '1');
     SetupTelescope.Edit_V_Point_R_AD.text := ReadString('Parameters', 'V_Point_R_AD', '1');
     SetupTelescope.Edit_V_Point_R_DEC.text := ReadString('Parameters', 'V_Point_R_DEC', '1');
     SetupTelescope.SpinEdit_A_Guidage_AD.value := ReadInteger('Parameters', 'A_Guidage_AD', 25);
     SetupTelescope.SpinEdit_A_Guidage_DEC.value := ReadInteger('Parameters', 'A_Guidage_DEC', 25);
     SetupTelescope.SpinEdit_A_Lent_AD.value := ReadInteger('Parameters', 'A_Lent_AD', 25);
     SetupTelescope.SpinEdit_A_Lent_DEC.value := ReadInteger('Parameters', 'A_Lent_DEC', 25);
     SetupTelescope.SpinEdit_A_Rapide_AD.value := ReadInteger('Parameters', 'A_Rapide_AD', 25);
     SetupTelescope.SpinEdit_A_Rapide_DEC.value := ReadInteger('Parameters', 'A_Rapide_DEC', 25);
     SetupTelescope.ComboBox_Accel_AD.ItemIndex := strtoint(ReadString('Parameters', 'Accel_AD', '0'));
     SetupTelescope.ComboBox_Accel_DEC.ItemIndex := strtoint(ReadString('Parameters', 'Accel_DEC', '0'));
    end;
    DeviceIni.free;

     If (SetupTelescope.ShowModal=MrOK) then     // Uniquement si *Validation* !
     begin
      DeviceIni := TIniFile.Create('mcmtii.ini');
      with DeviceIni do
        begin
         NumCom := StrToInt(SetupTelescope.ComboBoxNumCOM.text);
         WriteInteger('Configuration','Port',NumCOM);
         WriteString('Configuration','FirmwareFile',SetupTelescope.OpenDialog1.filename);
         WriteBool('Parameters', 'DIR_AD', SetupTelescope.CheckBox_DIR_AD.Checked);
         WriteBool('Parameters', 'DIR_DEC', SetupTelescope.CheckBox_DIR_DEC.checked);
         WriteBool('Parameters', 'PEC_AD', SetupTelescope.CheckBox_PEC.checked);
         WriteBool('Parameters', 'Raq_Can', SetupTelescope.CheckBox_Raq_Can.checked);
         Inv_Codeur_AD := SetupTelescope.CheckBox_DIR_AD.checked;
         Inv_Codeur_DEC := SetupTelescope.CheckBox_DIR_DEC.checked;
         Resol_AD := strtoint(SetupTelescope.Edit_Resol_AD.text);
         WriteInteger('Parameters', 'Resol_AD', Resol_AD);
         Resol_DEC := strtoint(SetupTelescope.Edit_Resol_DEC.text);
         WriteInteger('Parameters', 'Resol_DEC', Resol_DEC);
         WriteString('Parameters', 'ReducAlpha', SetupTelescope.EditResolAlpha.text);
         WriteString('Parameters', 'ReducDelta', SetupTelescope.EditResolDelta.text);
         WriteString('Parameters', 'V_Guidage_AD', SetupTelescope.Edit_V_Guidage_AD.text);
         WriteString('Parameters', 'V_Guidage_DEC', SetupTelescope.Edit_V_Guidage_DEC.text);
         WriteString('Parameters', 'V_Corr_P_AD', SetupTelescope.Edit_V_Corr_P_AD.text);
         WriteString('Parameters', 'V_Corr_P_DEC', SetupTelescope.Edit_V_Corr_P_DEC.text);
         WriteString('Parameters', 'V_Corr_M_AD', SetupTelescope.Edit_V_Corr_M_AD.text);
         WriteString('Parameters', 'V_Corr_M_DEC', SetupTelescope.Edit_V_Corr_M_DEC.text);
         WriteString('Parameters', 'V_Point_L_AD', SetupTelescope.Edit_V_Point_L_AD.text);
         WriteString('Parameters', 'V_Point_L_DEC', SetupTelescope.Edit_V_Point_L_DEC.text);
         WriteString('Parameters', 'V_Point_R_AD', SetupTelescope.Edit_V_Point_R_AD.text);
         WriteString('Parameters', 'V_Point_R_DEC', SetupTelescope.Edit_V_Point_R_DEC.text);
         WriteInteger('Parameters', 'A_Guidage_AD', SetupTelescope.SpinEdit_A_Guidage_AD.value);
         WriteInteger('Parameters', 'A_Guidage_DEC', SetupTelescope.SpinEdit_A_Guidage_DEC.value);
         WriteInteger('Parameters', 'A_Lent_AD', SetupTelescope.SpinEdit_A_Lent_AD.value);
         WriteInteger('Parameters', 'A_Lent_DEC', SetupTelescope.SpinEdit_A_Lent_DEC.value);
         WriteInteger('Parameters', 'A_Rapide_AD', SetupTelescope.SpinEdit_A_Rapide_AD.value);
         WriteInteger('Parameters', 'A_Rapide_DEC', SetupTelescope.SpinEdit_A_Rapide_DEC.value);
         WriteString('Parameters', 'Accel_AD', inttostr(SetupTelescope.ComboBox_Accel_AD.ItemIndex));
         WriteString('Parameters', 'Accel_DEC', inttostr(SetupTelescope.ComboBox_Accel_DEC.ItemIndex));
        end;
      DeviceIni.free;
      V_Guidage_AD := strtofloat(SetupTelescope.Edit_V_Guidage_AD.text);
      V_Guidage_AD_SID := V_Guidage_AD;
      V_Guidage_DEC := strtofloat(SetupTelescope.Edit_V_Guidage_DEC.text);
      V_Corr_P_AD := strtofloat(SetupTelescope.Edit_V_Corr_P_AD.text);
      V_Corr_P_DEC := strtofloat(SetupTelescope.Edit_V_Corr_P_DEC.text);
      V_Corr_M_AD := strtofloat(SetupTelescope.Edit_V_Corr_M_AD.text);
      V_Corr_M_DEC := strtofloat(SetupTelescope.Edit_V_Corr_M_DEC.text);
      V_Point_L_AD := strtofloat(SetupTelescope.Edit_V_Point_L_AD.text);
      V_Point_L_DEC := strtofloat(SetupTelescope.Edit_V_Point_L_DEC.text);
      V_Point_R_AD := strtofloat(SetupTelescope.Edit_V_Point_R_AD.text);
      V_Point_R_DEC := strtofloat(SetupTelescope.Edit_V_Point_R_DEC.text);
      Accel_AD := SetupTelescope.ComboBox_Accel_AD.ItemIndex;
     end;
    SetupTelescope.Free; //Destroy...
//  end;
end;

// very usefull function that allow to know if an axis is slewing or not!
// especially written for the ASCOM driver
Function IsAxisSlewing(Axis:integer):boolean;stdcall;
const Alpha=0;
      Decli=1;
var
   RecBuff : array [0..7] of byte;
   SendBuff : array [0..3] of byte;
   abyte,numbyte,retry: byte;
   NbOfBytesWrite,NbOfBytesRead:LongWord;
   deviceready:integer;
begin
   result:=False;
//   Open_Cod_Eng(Err_COM);  // ouvrir le port COM
   FlushFileBuffers(hCom);
   numbyte := 2;
   SendBuff[0] := AxeZero + Axis;
   SendBuff[1] := 255;
   retry := 3;
   repeat
      WriteFile(hCom,SendBuff,numbyte,NbOfBytesWrite,nil);
      ReadFile(hCom,RecBuff[0],1,NbOfBytesRead,nil);
      abyte := RecBuff[0];
      retry := retry -1;
//      Application.ProcessMessages;
   until ( retry=0 ) or (NbOfBytesRead <> 0);
   if abyte = 1 then deviceready := 1
   else deviceready := 0;
   if(NbOfBytesRead = 0) then
      deviceready := -1;
   if deviceready=0 then result:=True;
//   CloseHandle(hCom);
end;

// allow driver for ASCOM to get the different speeds in radian/sec
procedure GetSpeed(var Guidage_AD,Guidage_DEC,Corr_P_AD,Corr_P_DEC,
                  Corr_M_AD,Corr_M_DEC,Point_L_AD,Point_L_DEC,
                  Point_R_AD,Point_R_DEC : double);stdcall;
begin
    Guidage_AD := V_Guidage_AD/Resol_AD*2*PI;
    Guidage_DEC := V_Guidage_DEC/Resol_DEC*2*PI;
    Corr_P_AD := V_Corr_P_AD/Resol_AD*2*PI;
    Corr_P_DEC := V_Corr_P_DEC/Resol_DEC*2*PI;
    Corr_M_AD := V_Corr_M_AD/Resol_AD*2*PI;
    Corr_M_DEC := V_Corr_M_DEC/Resol_DEC*2*PI;
    Point_L_AD := V_Point_L_AD/Resol_AD*2*PI;
    Point_L_DEC := V_Point_L_DEC/Resol_DEC*2*PI;
    Point_R_AD := V_Point_R_AD/Resol_AD*2*PI;
    Point_R_DEC := V_Point_R_DEC/Resol_DEC*2*PI;
end;


//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
//////// ///////// ///////// ///////// ///////// ///////// ///////// /
(* These functions MUST be exported to Prism as it is *)

exports
 //General
 GetDLLVer  name 'GetDLLVer',

 //Encoders
 InitCodeurs  name 'InitCodeursDevice_DLL',
 CloseCodeurs name 'CloseCodeursDevice_DLL',
 Calibrate    name 'CalibrateCodeursDevice_DLL',
 GetAlpha     name 'GetAlphaCodeursDevice_DLL',
 GetDelta     name 'GetDeltaCodeursDevice_DLL',
 RetrieveDataAlphaDelta name 'CheckCodeursCodeursDevice_DLL',
 GetRAWAlphaDelta  name 'GetRAWAlphaDeltaCodeursDevice_DLL',
 GetRawPositionAlphaInit_arcsecCodeurs name 'GetRawPositionAlphaInit_arcsecCodeursDevice_DLL',
 GetRawPositionDeltaInit_arcsecCodeurs name 'GetRawPositionDeltaInit_arcsecCodeursDevice_DLL',
 GetRawPositionAlpha_arcsecCodeurs name 'GetRawPositionAlpha_arcsecCodeursDevice_DLL',
 GetRawPositionDelta_arcsecCodeurs name 'GetRawPositionDelta_arcsecCodeursDevice_DLL',

 //Telescope Slewing

 InitTelescopeSeeker_DLL               name  'InitTelescopeSeekerDevice_DLL',
 Execute_TelescopeSeeker_DLL           name  'Execute_TelescopeSeekerDevice_DLL',
 StopSlewing_TelescopeSeekerDevice_DLL name  'StopSlewing_TelescopeSeekerDevice_DLL',
 CloseTelescopeSeeker_DLL              name  'CloseTelescopeSeekerDevice_DLL',

 // telescope Engine -> Raquette
 InitTelescopeEngine_DLL   name 'InitTelescopeEngineDevice_DLL',
 GetSpeedAlpha             name 'GetSpeedAlphaEngineDevice_DLL',
 GetSpeedDelta             name 'GetSpeedDeltaEngineDevice_DLL',
 MoveOffsetDirectAD        name 'MoveOffsetDirectADEngineDevice_DLL',
 MoveTelescope             name 'MoveEngineDevice_DLL',
 MoveEventTelescope        name 'MoveEventEngineDevice_DLL',
 STOPTelescope             name 'StopTelescopeEngineDevice_DLL',
 STOP_ALL_Telescope        name 'Stop_ALL_TelescopeEngineDevice_DLL',
 ReturnToSideralSpeed      name 'ReturnToSideralSpeedEngineDevice_DLL',
 SetFreeSpeed              name 'SetFreeSpeedDevice_DLL',
 StopFreeSpeed             name 'StopFreeSpeedDevice_DLL',
 StartSolarEngine          name 'StartSolarSpeedEngineDevice_DLL',
 StartMoonEngine           name 'StartMoonSpeedEngineDevice_DLL',
 ShowConsoleEngine         name 'ShowConsoleEngineDevice_DLL',
 ShowSpeedEngine           name 'ShowSpeedEngineDevice_DLL',
 CloseEngine               name 'CloseEngineTelescopeDevice_DLL',

 GetCurrentSpeedRA         name 'GetCurrentSpeedRADevice_DLL',
 GetCurrentSpeedDEC        name 'GetCurrentSpeedDECDevice_DLL',

 PulseGuide                name 'PulseGuideDevice_DLL',
 //King

 StartKingDevice           name 'StartKingDevice_DLL',
 StopKingDevice           name 'StopKingDevice_DLL',

 // PEC
 AcceptRawPositionCodeurs  name 'AcceptRawPositionCodeursDevice_DLL',
 StartPecSpeedEngine       name 'StartPecSpeedEngineDevice_DLL',
 StopPec                   name 'StopPec_DLL',

 // PANEL d'INIT
 Init_telescope_Panel      name 'Init_telescope_Panel_DLL',

 // FOR ASCOM
 IsAxisSlewing             name 'IsAxisSlewing_DLL',
 GetSpeed                  name 'GetSpeed_DLL';
 //Focusing

 //Dome

begin
end.










